﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;
using System.Windows.Documents;
using System.Windows.Media;
using System.Collections.Generic;

namespace VectorGrafik
{
    public class ResizeThumb : Thumb
    {
        private DesignerItem designerItem;
        private DesignerCanvas designerCanvas;
        private Adorner adorner;
        private double angle;
        private Point transformOrigin;

        public ResizeThumb()
        {
            DragStarted += new DragStartedEventHandler(this.ResizeThumb_DragStarted);
            DragDelta += new DragDeltaEventHandler(this.ResizeThumb_DragDelta);
            DragCompleted += new DragCompletedEventHandler(this.ResizeThumb_DragCompleted);
        }

        private void ResizeThumb_DragStarted(object sender, DragStartedEventArgs e)
        {
            this.designerItem = DataContext as DesignerItem;
            if (this.designerItem != null)
            {
                this.transformOrigin = this.designerItem.RenderTransformOrigin;
                RotateTransform rotateTransform = this.designerItem.RenderTransform as RotateTransform;
                this.designerCanvas = VisualTreeHelper.GetParent(this.designerItem) as DesignerCanvas;
                AdornerLayer adornerLayer = AdornerLayer.GetAdornerLayer(this.designerCanvas);
                foreach (DesignerItem item in this.designerCanvas.SelectedItems)
                {
                    if (adornerLayer != null)
                    {
                        this.adorner = new SizeAdorner(item);
                        adornerLayer.Add(this.adorner);
                    }
                }
                if (rotateTransform != null)
                {
                    this.angle = rotateTransform.Angle * Math.PI / 180.0;
                }
                else
                {
                    this.angle = 0;
                }
            }
        }

        private void ResizeThumb_DragDelta(object sender, DragDeltaEventArgs e)
        {
            if (this.designerItem != null)
            {
                double deltaVertical, deltaHorizontal;

                foreach (DesignerItem item in this.designerCanvas.SelectedItems)
                {

                    switch (VerticalAlignment)
                    {
                        case System.Windows.VerticalAlignment.Bottom:
                            deltaVertical = Math.Min(-e.VerticalChange, item.ActualHeight - item.MinHeight);
                            Canvas.SetTop(item, Canvas.GetTop(item) + (this.transformOrigin.Y * deltaVertical * (1 - Math.Cos(-this.angle))));
                            Canvas.SetLeft(item, Canvas.GetLeft(item) - deltaVertical * this.transformOrigin.Y * Math.Sin(-this.angle));
                            item.Height -= deltaVertical;
                            break;
                        case System.Windows.VerticalAlignment.Top:
                            deltaVertical = Math.Min(e.VerticalChange, item.ActualHeight - item.MinHeight);
                            Canvas.SetTop(item, Canvas.GetTop(item) + deltaVertical * Math.Cos(-this.angle) + (this.transformOrigin.Y * deltaVertical * (1 - Math.Cos(-this.angle))));
                            Canvas.SetLeft(item, Canvas.GetLeft(item) + deltaVertical * Math.Sin(-this.angle) - (this.transformOrigin.Y * deltaVertical * Math.Sin(-this.angle)));
                            item.Height -= deltaVertical;
                            break;
                        default:
                            break;
                    }

                    switch (HorizontalAlignment)
                    {
                        case System.Windows.HorizontalAlignment.Left:
                            deltaHorizontal = Math.Min(e.HorizontalChange, item.ActualWidth - item.MinWidth);
                            Canvas.SetTop(item, Canvas.GetTop(item) + deltaHorizontal * Math.Sin(this.angle) - this.transformOrigin.X * deltaHorizontal * Math.Sin(this.angle));
                            Canvas.SetLeft(item, Canvas.GetLeft(item) + deltaHorizontal * Math.Cos(this.angle) + (this.transformOrigin.X * deltaHorizontal * (1 - Math.Cos(this.angle))));
                            item.Width -= deltaHorizontal;
                            break;
                        case System.Windows.HorizontalAlignment.Right:
                            deltaHorizontal = Math.Min(-e.HorizontalChange, item.ActualWidth - item.MinWidth);
                            Canvas.SetTop(item, Canvas.GetTop(item) - this.transformOrigin.X * deltaHorizontal * Math.Sin(this.angle));
                            Canvas.SetLeft(item, Canvas.GetLeft(item) + (deltaHorizontal * this.transformOrigin.X * (1 - Math.Cos(this.angle))));
                            item.Width -= deltaHorizontal;
                            break;
                        default:
                            break;
                    }
                }
            }

            e.Handled = true;

        }

        private void ResizeThumb_DragCompleted(object sender, DragCompletedEventArgs e)
        {
            AdornerLayer adornerLayer = AdornerLayer.GetAdornerLayer(this.designerCanvas);
            foreach (DesignerItem item in this.designerCanvas.SelectedItems)
            {
                Adorner[] adorners = adornerLayer.GetAdorners(item);
                Adorner toRemove = adorners[1];
                adornerLayer.Remove(toRemove);

            }
           
            this.adorner = null;
            
        }
    }
}
